;
;
;+
; NAME:
;       ASPECT
;
; PURPOSE:
;       This function calculates and returns the normalized position
;       coordinates necessary to put a plot with a specified aspect ratio
;       into the currently active graphics window. It works on the display
;       output window as well as in a PostScript output window.
;
; CATEGORY:
;       Graphics
;
; CALLING SEQUENCE:
;
;       position = ASPECT(aspectRatio)
;
; INPUTS:
;       aspectRatio: A floating point value that is the desired aspect
;       ratio (ratio of heigth to width) of the plot in the current
;       graphics output window. If this parameter is missing, an aspect
;       ratio of 1.0 (a square plot) is assumed.
;
; KEYWORD PARAMETERS:
;       MARGIN: The margin around the edges of the plot. The value must be
;       a floating point value between 0.0 and 0.5. It is expressed in
;       normalized coordinate units. The default margin is 0.15.
;
; OUTPUTS:
;       position: A four-element floating array of normalized coordinates.
;       The order of the elements is [x0, y0, x1, y1], similar to the
;       !P.POSITION system variable or the POSITION keyword on any IDL
;       graphic command.
;
; EXAMPLE:
;       To create a plot with an aspect ratio of 1:2 and a margin of
;       0.10 around the edge of the output window, do this:
;
;          plotPosition = ASPECT(0.5, Margin=0.10)
;          PLOT, Findgen(11), POSITION=plotPosition
;
;       Notice this can be done in a single IDL command, like this:
;
;          PLOT, Findgen(11), POSITION=ASPECT(0.5, Margin=0.10)
;
; MODIFICATION HISTORY:
;       Written by:     David Fanning, November 1996.
;       Added better error checking, 18 Feb 97, DWF.
;-

FUNCTION ASPECT, aspectRatio, MARGIN=margin

  ON_ERROR, 1

   ; Check for aspect ratio parameter and possibilities.

  IF N_PARAMS() EQ 0 THEN aspectRatio = 1.0

  IF aspectRatio EQ 0 THEN BEGIN
   MESSAGE, 'Aspect Ratio of 0. Changing to 1...', /Informational
   aspectRatio = 1.0
  ENDIF

  s = SIZE(aspectRatio)
  IF s(s(0)+1) NE 4 THEN $
   MESSAGE, 'Aspect Ratio is not a FLOAT. Take care...', /Informational

   ; Check for margins.

  IF N_ELEMENTS(margin) EQ 0 THEN margin = 0.15

   ; Error checking.

  IF margin LT 0 OR margin GE 0.5 THEN $
   MESSAGE, 'The MARGIN keyword value must be between 0.0 and 0.5.'

   ; Calculate the aspect ratio of the current window.

  wAspectRatio = FLOAT(!D.Y_VSIZE) / !D.X_VSIZE

   ; Calculate normalized positions in window.

  IF (aspectRatio LE wAspectRatio) THEN BEGIN
   xstart = margin
   ystart = 0.5 - (0.5 - margin) * (aspectRatio / wAspectRatio)
   xend = 1.0 - margin
   yend = 0.5 + (0.5 - margin) * (aspectRatio / wAspectRatio)
  ENDIF ELSE BEGIN
   xstart = 0.5 - (0.5 - margin) * (wAspectRatio / aspectRatio)
   ystart = margin
   xend = 0.5 + (0.5 - margin) * (wAspectRatio / aspectRatio)
   yend = 1.0 - margin
  ENDELSE

  position = [xstart, ystart, xend, yend]

RETURN, position
END





PRO PLOT_IT, mstruc, $
	     datstruc, $
	     whichdata, $
	     XTitle, $
	     XValue = value, $
	     xtype, $
	     XR, $
	     YR

	IF !D.NAME NE 'TEK' THEN BEGIN
	   IF !D.WINDOW GE 0 THEN $
		WSHOW, 0
	ENDIF

	IDATE = SYSTIME()
	!p.font = 0
;
;	Maximum of 7 spectra, therefore 7 colors + background
;       back, 'Spec' 'Fit' 'All Bnds' 'Ind. Bnds' 'Cont' 'Spec-Cont' 'Residual' 

	r = [0, 1, 1,0,0,1,0,1]*255
	g = [0, 0, 1,1,0,0,1,1]*255
	b = [0, 1, 1,0,1,0,1,0]*255

;	lines =  [0,2,1,3,4,5,2]
	lines =  [0,0,3,0,5,1,0]
	pthick = [1,2,1,1,1,1,1]
	symbols =[1,0,0,0,0,0,0]

	IF !D.N_COLORS GT 256 THEN BEGIN
		cindex = r(whichdata) + 256L * $
		         ( g(whichdata)+256L * b(whichdata) )
	ENDIF ELSE BEGIN
		cindex = INDGEN(7)+1
	ENDELSE

	IF NOT KEYWORD_SET( value ) THEN $
		value = datstruc.wavel/1000.		;default x-axis
	IF NOT KEYWORD_SET( XRange ) THEN $
		XRange = [mstruc.XRange(0),mstruc.XRange(1)]
	IF NOT KEYWORD_SET( YRange ) THEN $
		YRange = [mstruc.YRange(0),mstruc.YRange(1)]

	ppos = ASPECT( (5.95/8.5), MARGIN=0.125 )
;	ppos([0,2]) = [ppos([0,2])-0.05]
;	ppos = [0.15,0.1,0.75,0.9]

	IF xtype EQ 2 THEN Xrange = [XR(3), XR(2)] $
		ELSE Xrange = [XR(0), XR(1)]
	PLOT, [0.], [0.], $
		    /NODATA, $
		    XTITLE = xtitle, $
		    YTITLE = ' Natural Log Reflectance', $
		    TITLE  = mstruc.datfile, $
		    SUBTITLE = idate, $
		    YRANGE = YR, $
		    XRANGE = Xrange,  $
		    XSTYLE = 1, $
		    YSTYLE = 1, $
		    POSITION = ppos	

	xpos = ppos(2)+0.1
	xpos2= xpos - 0.075

	ndx = where( whichdata GT 0, nspec )
	ypos = FLTARR(nspec)
	ypos(0) = ppos(3)

	FOR i = 1,nspec-1 DO $
	    ypos(i) = ypos(i-1) - 0.05 

	FOR i = 0,nspec-1 DO BEGIN
	    CASE whichdata( ndx(i) ) OF 
	    1: BEGIN
		y = datstruc.ratio
;		klabel = 'Spectra'	    	
		errors=WHERE (datstruc.daterror NE 1)
		IF errors(0)  NE -1 THEN BEGIN
		  errors = (datstruc.daterror)	
		  ERRPLOT, value, (y-errors), (y+errors)
		ENDIF	
	       END
	    2: BEGIN
	        y = datstruc.fit
;		klabel = 'Fit'
	       END
	    3: BEGIN
		y = datstruc.gline
;		klabel = 'All Bnds'
	       END
	    4: BEGIN
		y = datstruc.gauss
;		klabel = 'Ind. Bnds'
	       END
	    5: BEGIN
		y = datstruc.cont
;		klabel = 'Cont'
	       END
	    6: BEGIN
		y = datstruc.rcont
;		klabel = 'Spec-Cont'
	       END
	    7:  BEGIN
		y = datstruc.resid+0.10
;		klabel = 'Residual'
	       END
	    ELSE:
	    ENDCASE

	    sz = SIZE( y )
	    IF sz(0) GT 1 THEN $
		b = sz(sz(0)) ELSE $
		b = 1
		
	    FOR kk = 0, b-1 DO BEGIN
	    	OPLOT, value, y(*,kk), $
		       LINES=lines(ndx(i)), $
		       COLOR=cindex(ndx(i)), $
		       THICK = pthick(ndx(i)), $
		       PSYM= symbols(ndx(i)), $
		       SYMSIZE = .5	
	    ENDFOR

;	    PLOTS, [xpos2,xpos2+0.05], $
;		   [ypos(i),ypos(i)], $
;		   COLOR=cindex(ndx(i)), $
;		   LINESTYLE=lines(ndx(i)),$
;		   THICK = pthick(ndx(i)), $
;		   /NORMAL

;    	    XYOUTS, xpos, ypos(i), klabel, /NORMAL, CHARSIZE=0.75
			
	ENDFOR
	RETURN
	END



PRO DISPLAY, mstruc, datstruc

;	GRAPHICS set up...

	
	mydevice = !D.NAME
	SET_PLOT, mydevice

	idate = SYSTIME()

	IF mydevice NE 'TEK' THEN BEGIN
	  IF !D.WINDOW GE 0 THEN $
		WSHOW, 0 ELSE $
		WINDOW, XSIZE=850, YSIZE=595, RETAIN=2
	ENDIF

	whichdata = [1,2,0,4,5,0,7]  ;default initial curves (s,f,i,c,e)

	xtype = 0
	xtitle = 'Wavelength (Microns)'
	YR = [mstruc.YRange(0), mstruc.YRange(1)]
	XMin = MIN( datstruc.waven/1000., MAX=XMax)
	XR = [mstruc.XRange(0), mstruc.XRange(1), XMin, XMax]
	value = datstruc.wavel/1000.
	
	PLOT_IT, mstruc, $
	     datstruc, $
	     whichdata, $
	     XTitle, $
	     XValue = value, $
	     xtype, $
	     XR, $
	     YR

	i1 = ''
	IF mydevice EQ 'TEK' THEN BEGIN
		READ, i1		;pause so that user may return to 
					;text window.  otherwise, the menu
					;will overwrite itself on the plot.
	ENDIF

	YesDisplay = 1

	WHILE YesDisplay DO BEGIN

	   yesps = 0

	   PRINT, IDATE, $
	   FORMAT = "(//1X,A,8X,'=====  MGM-FIT  Display Routines =====',//)"
;
;	   write menu
;
	   PRINT,'   Select: 1-Display Graph in Wavelength Space'
	   PRINT,'           2-Display Graph in Wavenumber Space'
	   PRINT,'           3-Reset Display Ranges'
	   PRINT,'           4-Reselect Curves'
	   PRINT,'           5-Send to Encapsulted PostScript'
	   PRINT,'           6-Send to TIFF File'
	   PRINT,'           7-Exit Display      '

	   I1 = ''
	   READ, I1
	   IF (I1 EQ '') THEN RETURN
	
	   CASE FIX(I1) OF
	   1: BEGIN
	   	value = datstruc.wavel/1000.
	   	xtitle = ' Wavelength (Microns)'
		xtype = 0
	      END
	   2: BEGIN
	   	value = datstruc.waven/1000.
	   	xtitle = ' Wavenumber (cm-1/1000)'
	   	xtype = 2
	      END
	   3: BEGIN
;	
;	      WRITE TEXT ON SCREEN
;
		xmin = ' '
	   	READ, xmin, PROMPT=' Enter MINIMUM Wavelength Range: '
	   	IF STRLEN(xmin) GT 0. THEN $
	   	   XR(xtype) = xmin
		xmax = ' '
	   	READ, xmax, PROMPT=' Enter MAXIMUM Wavelength Range: '
	   	IF STRLEN(xmax) GT 0. THEN $
		   XR(xtype +1) = xmax
		ymin = ' '
	   	READ, ymin, PROMPT=' Enter MINIMUM Log Reflectance Range: '
	   	IF STRLEN(ymin) GT 0. THEN $
	   	   YR(0) = ymin
		ymax = ' '
	   	READ, ymax, PROMPT=' Enter MAXIMUM Log Reflectance Range: '
	   	IF STRLEN(ymax) GT 0. THEN $
	   	   YR(1) = ymax
	      END
	   4: BEGIN
;
;	      FIND OUT WHICH CURVES TO DISPLAY
;
	   	PRINT, ' Select Curves to be Displayed'
	   	PRINT, '   S(pectra),B(ands),I(ndivid. Bands), R(Spec-Cont)'
	   	PRINT, '   F(it),C(ontinuum),E(Spec-Fit),A(All of them): '

	   	answers = ''
	   	READ, ANSWERS
	     	ANSWERS = STRTRIM( ANSWERS,2 )
	   	IF ( STRLEN(STRMID(ANSWERS,0,1)) EQ 0 ) THEN $
		      answers='A'
;
;	   	CLEAR DISPLAY FLAGS
;
	   	whichdata = INTARR(7)
	   	ichar = STRLEN( Answers )
	   	FOR k = 0,ichar-1 DO BEGIN
	             CASE STRUPCASE( STRMID(Answers,k,1) ) OF
	    	     'S': whichdata(0) = 1		; SPECTRA
	    	     'F': whichdata(1) = 2		; FIT
	    	     'B': whichdata(2) = 3		; COMBINED BANDS
	    	     'I': whichdata(3) = 4		; INDIVIDUAL BANDS
	    	     'C': whichdata(4) = 5		; CONTINUUM
	    	     'R': whichdata(5) = 6		; REMAIN (SPEC-CONT)
	    	     'E': whichdata(6) = 7		; RESIDUAL (SPEC-FIT)
		     'A': whichdata = INDGEN(7)+1	; ALL OF THEM
	    	     ELSE:
	             ENDCASE
	   	ENDFOR
	      END
	   5: BEGIN
	   	ipos = STRPOS( mstruc.datfile, '.asc' )
	   	postfile = STRMID( mstruc.datfile, 0, ipos )
	   	SET_PLOT, 'PS'
	   	DEVICE, FILE=postfile + '.EPS', /LANDSCAPE, /ENCAP
	   	yesps = 1
	      END
	   6: BEGIN
	   	ipos = STRPOS( mstruc.datfile, '.asc' )
	   	tifffile = STRMID( mstruc.datfile, 0, ipos ) + '.TIF'		
		WRITE_TIFF, tifffile, TVRD()
	      END
	   7: YesDisplay = 0
	   ELSE:
	   ENDCASE

	   IF YesDisplay THEN BEGIN
                PLOT_IT, mstruc, $
                        datstruc, $
                        whichdata, $
                        XTitle, $
                        XValue = value, $
			xtype, $
			XR, $
			YR
	   ENDIF

	   IF yesps GT 0 THEN BEGIN
	   	DEVICE, /CLOSE
		SET_PLOT, mydevice
	   ENDIF

	   IF mydevice EQ 'TEK' THEN BEGIN
		READ, i1		;pause so that user may return to 
					;text window.  otherwise, the menu
					;will overwrite itself on the plot.
	   ENDIF

	ENDWHILE

	RETURN
	END
