FUNCTION READ_FITFILE, FitFile

; This routine reads a FIT file with the following formatted data:
;
;	directory
;	file
;	ipmin,ipmax,ivtic   for  display
;	ihmin,ihmax,ihtic   for display
;	baseline constant
;	type of continuum   g gauss  p polynomial   n none
;       p1,p2,p3  for continuum
;	number of bands
;	center in nm   fwhm in inverse cm    intensity
;	center in nm   fwhm in inverse cm    intensity
;	center in nm   fwhm in inverse cm    intensity
;	etc
;
; INPUTS:
;     FitFile:	String defining FIT file to read
;
; OUTPUTS:
;     MStruc: 	IDL structure containing initial parameters for fit.
;

openr, lun, FitFile, /get_lun

;ext  = '.dat'
ext  = '.asc'
line = ''

; read in directory path
readf, lun, line
direct = line
ipos = STRPOS( direct, '[' )
IF ipos GT 0 THEN BEGIN
	PRINT, 'WARNING!  Directory specification is VMS'
	PRINT, '          Ignoring....'
	direct = ''
ENDIF

; read in file name
readf, lun, line
filename = line
datafile = strtrim(direct,2) + strtrim(filename,2) + ext

; read in y-axis range
ipmin = 0
ipmax = 0
ivtic = 0
readf, lun, line
reads, line, ipmin, ipmax, ivtic
YRange = [ipmin,ipmax,ivtic]

; read in x-axis range
iwmin = 0
iwmax = 0
ihtic = 0
readf, lun, line
reads, line, iwmin, iwmax, ihtic
XRange = [iwmin,iwmax,ihtic]

XRange = XRange/1000.
YRange = YRange/100.

; read in error limits
rmslim = 0.
rimplim = 0.
readf, lun, line
reads, line, rmslim, rimplim

; read in continuum type
readf, lun, line
contyp = STRTRIM(STRUPCASE(line),2)

; read in initial 'c' parameters
readf, lun, line
cp = fltarr(4)
reads, line, cp

; read in initial 's' parameters
readf, lun, line
sp = fltarr(4)
reads, line, sp

; read in initial gaussian paramters (3 per gaussian dist.)
readf, lun, line
ngauss = fix(line)	; the number of gaussians in fit

gcent = fltarr(ngauss)
gfwhm = fltarr(ngauss)
gstr  = fltarr(ngauss)
scent = fltarr(ngauss)
sfwhm = fltarr(ngauss)
sstr  = fltarr(ngauss)

center = 0.
fwhm   = 0.
amp    = 0.

for k = 0,ngauss-1 do begin
	readf, lun, line
	reads, line, center, fwhm, amp
	gcent(k) = center
	gfwhm(k) = fwhm
	gstr(k)  = amp

	readf, lun, line
	reads, line, center, fwhm, amp
	scent(k) = center
	sfwhm(k) = fwhm
	sstr(k)  = amp
endfor
close, lun
free_lun, lun

IF ( contyp EQ 'S' ) THEN BEGIN
   sWav1 = cp(0)
   sWav2 = cp(1)
ENDIF ELSE $
IF ( contyp EQ 'T' ) THEN BEGIN
   sWav1 = cp(0)
   sWav2 = cp(1)
ENDIF ELSE BEGIN
   sWav1 = 0.
   sWav2 = 0.
ENDELSE

nparam = 3*ngauss + 4

roots  = { cnrt:  fltarr(ngauss), $
	   sgsq:  fltarr(ngauss) }

mstruc = { FITfile: Fitfile, $
	   DATdir: direct, $
	   DFile: filename, $
	   DATfile: datafile, $
	   wconst: 2.354820, $
	   contyp: contyp, $
	   cparam: cp, $
	   sparam: sp, $
	   sWav1: sWav1, $
	   sWav2: sWav2, $
	   rmslim: rmslim, $
	   rimplim: rimplim, $
	   gcent: gcent, $
	   gfwhm: gfwhm, $
	   gstr: gstr, $
	   scent: scent, $
	   sfwhm: sfwhm, $
	   sstr: sstr, $
	   gcentn: fltarr(ngauss), $
	   gwidth: fltarr(ngauss), $
	   params: fltarr(nparam), $
	   nparam: nparam, $
	   nbands: ngauss, $
	   cmm: fltarr(nparam), $
	   ipstat: intarr(nparam,2), $
	   roots: roots, $
	   XRange: XRange, $
	   YRange: YRange, $
	   param0: fltarr(nparam), $
	   std: fltarr(nparam) }

return, mstruc
end
